using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.Threading;
using System.Text.RegularExpressions;

namespace Cotyounoyume{
	public class ExternalLinkage
	{
        EBATCore core;
        ColortoneSub colortoneSub;
        PostMagicSub postMagicSub;
        VAMMoanSub   vamMoanSub;

        public bool InDLing_Colortone = false;
        public bool InDLing_PostMagic = false;
        public bool InDLing_VAMMoan = false;


        public ExternalLinkage(EBATCore core){
            this.core = core;
            colortoneSub = new ColortoneSub(this.core);
            postMagicSub = new PostMagicSub(this.core);
            vamMoanSub   = new VAMMoanSub(this.core);
        }

        public void OnLoadDone(){
            // Utility.Debug("OnLoadDone in ExternalLinkage");
            postMagicSub.OnLoadDone();
            vamMoanSub.OnLoadDone();
        }

		public void SetColorCorrect(bool isOn){ colortoneSub.SetColorCorrect(isOn); }
        public bool IsColortoneOn(){ return colortoneSub.IsColortoneOn(); }
        public bool ExistsColortoneInTheGame(){ return colortoneSub.ExistsColortoneInTheGame(); }

        public bool IsPostMagicOn(bool isDoFOn){ return postMagicSub.IsPostMagicOn(isDoFOn); }
		public void SetPostMagic(bool isOn){ postMagicSub.SetPostMagic(isOn); }
        public bool ExistPostMagicInTheGame(){ return postMagicSub.ExistPostMagicInTheGame(); }
        public bool IsDoFOn(){ return postMagicSub.IsDoFOn();}
        public void DoFOnOff(bool isOn){ postMagicSub.DoFOnOff(isOn); }
    
        public bool IsVAMMoanOn(){ return vamMoanSub.IsVAMMoanOn(); }
		public void SetVAMMoan(bool isOn){ vamMoanSub.SetVAMMoan(isOn); }
        public bool ExistVAMMoanInTheGame(){ return vamMoanSub.ExistVAMMoanInTheGame(); }
    }


    public class VAMMoanSub{
        public string vamMoanLabelName = "VAMMoan";
        // public string vamMoanPrefix = "MacGruber.PostMagic.";
        public string vamMoanName = "VAMMoanPlugin.VAMMoan";
        public string vamMoanScriptPath = "Custom\\Scripts\\VAMMoan\\VAMMoan.cslist"; 
        public string vamMoanSearchPattern = "*vammoan*";
        public EBATCore core;
        public VAMMoanSub(EBATCore core){
            this.core = core;
        }

        public void OnLoadDone(){
            core.externalLinkage.InDLing_VAMMoan = false;
        }

        public bool IsVAMMoanOn(){
            var kvp = Utility.GetAtomAndPluginPairFromAtoms(vamMoanName);
            if(kvp == null) return false;
            return Utility.GetOnOffPlugin(kvp?.Key, kvp?.Value);
        }

        public void SetVAMMoan(bool isOn){
            if(!isOn){
                SetVAMMoanOnOff(false);
            }else if(ExistVAMMoanInTheScene()){
                SetVAMMoanOnOff(true);
            }else if(ExistVAMMoanInTheGame()){
                LoadVAMMoan();
            }else{
                core.externalLinkage.InDLing_VAMMoan = true;
                core.worldUi.DLGuideForVAMMoan();
            }
        }

        public bool ExistVAMMoanInTheScene(){
            var kvp = Utility.GetAtomAndPluginPairFromAtoms(vamMoanName);
            return kvp != null;
        }

        public bool ExistVAMMoanInTheGame(){
            var url = GetVAMMoanURL();
            return url != "";
        }

        public void SetVAMMoanOnOff(bool isOn){
            var kvp = Utility.GetAtomAndPluginPairFromAtoms(vamMoanName);
            if(kvp == null) return;

            var normalized = Utility.NormalizePluginName(kvp?.Value);
            // Utility.Debug($"  hit!: postmagic plugin found in kvp.Key:{kvp?.Key}, kvp.Value:{kvp?.Value}, normalized:{normalized}");
            Utility.ToggleOnOffPlugin(core.atom, kvp?.Value, isOn);
        }

        public string GetVAMMoanURL(){
			var pattern = vamMoanSearchPattern;
			var path = vamMoanScriptPath;
			var url = Utility.GetPluginURL(pattern, path);
            return url;
        }

        public void LoadVAMMoan(){
			var pattern = vamMoanSearchPattern;
			var path = vamMoanScriptPath;
			var file = GetVAMMoanURL();
			Utility.LoadPluginIfNeeded(core.atom, pattern, path, vamMoanLabelName, num:97);        
        }
    }

    public class PostMagicSub{
        public string PostMagicLabelName = "MacGruber_PostMagic";
        public string PostMagicPrefix = "MacGruber.PostMagic.";
        public string PostMagicManagerName = "MacGruber.PostMagic.Manager";
        public string PostMagicScriptPath = "Custom\\Scripts\\MacGruber\\PostMagic\\MacGruber_PostMagic.cslist"; 
        public string PostMagicSearchPattern = "*PostMagic*";
        public string DoFName = "MacGruber.PostMagic.DepthOfField";
        public bool DoFOn = false;
        public EBATCore core;
        public PostMagicSub(EBATCore core){
            this.core = core;
        }

        public Dictionary<string, bool> PostMagicOnMap = new Dictionary<string, bool>(){
            // {"MacGruber.PostMagic.Manager", false},
            {"MacGruber.PostMagic.AntiAliasing", false}, 
            {"MacGruber.PostMagic.Bloom", false},
            {"MacGruber.PostMagic.Grain", false},
            {"MacGruber.PostMagic.ChromaticAberration", true},
            {"MacGruber.PostMagic.DepthOfField", true},
            {"MacGruber.PostMagic.MotionBlur", true},
            {"MacGruber.PostMagic.UserLUT", true},
            {"MacGruber.PostMagic.Vignette", true},
        };

        public void OnLoadDone(){
            if(Utility.HasThisPlugin(core.atom, PostMagicLabelName)){
                if(CheckPostMagicLoadStatusIsDefault()) ChangePostMagicLoadStatus();
            }
            core.externalLinkage.InDLing_PostMagic = false;
        }

        public bool CheckPostMagicLoadStatusIsDefault(){
            var list = Utility.GetPluginsByPattern(core.atom, PostMagicPrefix);
            var totalResult = false;
            foreach(var name in list){
                if(name.Contains(PostMagicManagerName)) continue;
                var result = Utility.GetOnOffPlugin(core.atom, name);
                totalResult = totalResult | result;
                var normalized = Utility.NormalizePluginName(name);
                // Utility.Debug($"name:{name}, normalized:{normalized}, result:{result}");
            }
            totalResult = !totalResult;
            // Utility.Debug($"totalResult:{totalResult}");

            return totalResult;
        }

        public void ChangePostMagicLoadStatus(){
            var list = Utility.GetPluginsByPattern(core.atom, PostMagicPrefix);
            foreach(var name in list){
                var normalized = Utility.NormalizePluginName(name);
                if(normalized == PostMagicManagerName) continue;
                if(!PostMagicOnMap.ContainsKey(normalized)) continue;
                var isOn = PostMagicOnMap[normalized];
                if(normalized == DoFName){
                    isOn = DoFOn;
                }
                Utility.ToggleOnOffPlugin(core.atom, name, isOn);
            }
        }

        public bool IsPostMagicOn(bool isDoFOn){
            var kvp = Utility.GetAtomAndPluginPairFromAtoms(PostMagicManagerName);
            if(kvp == null) return false;
            return DoFOn == isDoFOn && Utility.GetOnOffPlugin(kvp?.Key, kvp?.Value);
        }

        public void DoFOnOff(bool isOn){
            DoFOn = isOn;
        }

        public bool IsDoFOn(){
            return DoFOn;
        }

        public void SetPostMagic(bool isOn){
            if(!isOn){
                SetPostMagicOnOff(false);
            }else{
                if(ExistPostMagicInTheScene()){
                    SetPostMagicOnOff(true);
                }else if(ExistPostMagicInTheGame()){
                    LoadPostMagic();
                }else{
                    core.externalLinkage.InDLing_PostMagic = true;
                    core.worldUi.DLGuideForPostMagic();
                }
            }
        }

        public bool ExistPostMagicInTheScene(){
            var kvp = Utility.GetAtomAndPluginPairFromAtoms(PostMagicManagerName);
            return kvp != null;
        }

        public bool ExistPostMagicInTheGame(){
            var url = GetPostMagicURL();
            return url != "";
        }

        public void SetPostMagicOnOff(bool isOn){
            var kvp = Utility.GetAtomAndPluginPairFromAtoms(PostMagicManagerName);
            if(kvp == null) return;
            // var normalized = Utility.NormalizePluginName(kvp?.Value);
            Utility.ToggleOnOffPlugin(core.atom, kvp?.Value, isOn);

            kvp = Utility.GetAtomAndPluginPairFromAtoms(DoFName);
            if(kvp == null) return;
            // var normalized = Utility.NormalizePluginName(kvp?.Value);
            Utility.ToggleOnOffPlugin(core.atom, kvp?.Value, DoFOn);
        }

        public string GetPostMagicURL(){
			var pattern = PostMagicSearchPattern;
			var path = PostMagicScriptPath;
			var url = Utility.GetPluginURL(pattern, path);
            return url;
        }

        public void LoadPostMagic(){
			var pattern = PostMagicSearchPattern;
			var path = PostMagicScriptPath;
			var file = GetPostMagicURL();
			Utility.LoadPluginIfNeeded(core.atom, pattern, path, PostMagicLabelName, num:96);        
        }
    }

    public class ColortoneSub{
        public string ColortoneCUAName = "EBAT_Colortone";
        public string ColortoneAssetNamePath = "Assets/Scenes/TonemapOnly.unity";
        public string ColortoneAssetNameOnly = "TonemapOnly.unity";
        public string ColortoneAssetURLPath = "Custom/Assets/colorcorrect.assetbundle"; 
        public string ColortoneAssetFilename = "colorcorrect.assetbundle"; 
        public string ColortoneSearchPattern = "*TiSeb.Colortone*";
        public EBATCore core;
        public ColortoneSub(EBATCore core){
            this.core = core;
        }

        public bool IsColortoneOn(){
			var cua = GetColoCorrectAtom();
            if(cua == null) return false;
            return Utility.HasAssetName(cua, ColortoneAssetNameOnly);
        }

		public string GetColorcorrectURL(){
            var filename = Utility.GetLongestFilename(ColortoneSearchPattern);
            if(filename == "") return "";
            var varname = Utility.CorrectVarname(filename);
			return varname + ":/" + ColortoneAssetURLPath;
		}

		public void SetColorCorrect(bool isOn){
			if(!isOn) {
                SetColorCorrectOnOff(false);
            }else{
                if(ExistsColortoneInTheScene()){
                    SetColorCorrectOnOff(true);
                }else if(ExistsColortoneInTheGame()){
                    MakeCUAForColortone();
                }else{
                    core.externalLinkage.InDLing_Colortone = true;
                    core.worldUi.DLGuideForColortone();
                }
            }
		}

         public void MakeCUAForColortone(){
            var url = GetColorcorrectURL();
            core.customUnityAssetItem.SummonAtomMarker(ColortoneCUAName, "pelvisControl", 0f, -0.2f, 0f, bSetParent:true, debugOn:false);
            core.StartCoroutine(SetColorCorrectCoroutine(url));
        }

        public IEnumerator SetColorCorrectCoroutine(string url){
            Atom tmpatom = null;
            for(int i = 0; i < 200; i++){
                tmpatom = core.customUnityAssetItem.GetCUAAtom(ColortoneCUAName);
                if(tmpatom == null) yield return null;
            }
            if(tmpatom == null) yield break;

            Utility.SetAssetBundleUrl(tmpatom, url);
            Utility.ResetAssetBundleName(tmpatom, ColortoneAssetNamePath);
        }

        public bool ExistsColortoneInTheScene(){
            var cua = GetColoCorrectAtom();
            return cua != null;
        }

        public bool ExistsColortoneInTheGame(){
            var url = GetColorcorrectURL();
            return url != "";
        }

        public Atom GetColoCorrectAtom(){
            // return Utility.GetAssetIncludeAtom(ColortoneAssetURLPath);
            return Utility.GetAssetIncludeAtom(ColortoneAssetFilename);
        }

		public void SetColorCorrectOnOff(bool isOn){
			var cua = GetColoCorrectAtom();
             if(isOn){
    			Utility.ResetAssetBundleName(cua, ColortoneAssetNamePath);
            }else{
                if(cua == null) return;
                Utility.ResetAssetBundleName(cua, "");		
            }
		}
   }
}